import React, { useState } from "react";
import {
  Text,
  View,
  StyleSheet,
  TouchableOpacity,
  ActivityIndicator,
} from "react-native";
import PropTypes from "prop-types";

function DuolingoButton({
  style = {},
  title = "",
  textStyle = {},
  paddingStyle = { paddingVertical: 12, paddingHorizontal: 24 },
  primaryColor = "#F7BE1F",
  shadowColor = "#BF8E00",
  radius = 8,
  occupyFullWidth = false,
  Icon = undefined,
  onClickCallback,
  shadowDepth = 5,
  loaderColor = "#000000",
  loaderSize = 24,
  isLoading = false,
}) {
  const [isPress, setIsPress] = useState(false);

  const onPress = () => {
    setIsPress(true);
  };

  const onPressOut = () => {
    setIsPress(false);
    if (!isLoading) {
      onClickCallback();
    }
  };

  const onButtonLongPress = () => {
    setIsPress(false);
  };

  const isIconAvailable = Icon ? true : false;

  const justifyContentCenterStyle = {
    flex: 1,
    justifyContent: "center",
  };

  return (
    <TouchableOpacity
      activeOpacity={1}
      onPressIn={onPress}
      onPressOut={onButtonLongPress}
      onPress={onPressOut}
      style={{
        ...style,
        ...styles.appButtonDefault,
        ...(occupyFullWidth ? { width: "100%" } : {}),
        ...(!isPress ? styles.appButtonContainer : { paddingTop: shadowDepth }),
        ...(!isPress
          ? {
              backgroundColor: shadowColor,
              paddingBottom: shadowDepth,
            }
          : {}),
        borderRadius: radius,
      }}
    >
      <View
        style={{
          ...styles.actualButton,
          ...(occupyFullWidth ? justifyContentCenterStyle : {}),
          backgroundColor: primaryColor,
          ...(isIconAvailable ? styles.iconPositionStyle : {}),
          borderRadius: radius,
          ...paddingStyle,
        }}
      >
        <View style={{ ...{ opacity: isLoading ? 0 : 1 } }}>
          {isIconAvailable ? Icon : null}
        </View>
        <Text
          style={{
            ...styles.appButtonText,
            ...textStyle,
            ...{ opacity: isLoading ? 0 : 1 },
          }}
        >
          {title}
        </Text>
        {isLoading ? (
          <ActivityIndicator
            animating={isLoading}
            size={loaderSize}
            color={loaderColor}
            style={styles.buttonLoaderStyle}
          />
        ) : null}
      </View>
    </TouchableOpacity>
  );
}

const styles = StyleSheet.create({
  appButtonDefault: {
    radius: 10,
    alignSelf: "center",
    justifyContent: "center",
    alignItems: "center",
  },
  appButtonContainer: {
    paddingBottom: 5,
    backgroundColor: "green",
  },
  appButtonText: {
    fontSize: 24,
    color: "black",
    alignSelf: "center",
  },
  actualButton: {
    backgroundColor: "blue",
    radius: 8,
    width: "100%",
  },
  imageStyle: {
    width: 12,
    height: 12,
  },
  iconPositionStyle: {
    flexDirection: "row",
    alignItems: "center",
  },
  buttonLoaderStyle: {
    position: "absolute",
    alignItems: "center",
    justifyContent: "center",
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
  },
});
DuolingoButton.propTypes = {
  title: PropTypes.string.isRequired,
  textStyle: PropTypes.object,
  paddingStyle: PropTypes.object,
  primaryColor: PropTypes.string,
  shadowColor: PropTypes.string,
  radius: PropTypes.number,
  occupyFullWidth: PropTypes.bool,
  Icon: PropTypes.any,
  onClickCallback: PropTypes.func,
  shadowDepth: PropTypes.number,
};

export default DuolingoButton;
